require "toni/version"
require "toni/anonymous_user"
require "toni/permission_filter"
require "toni/builder"

module Toni
  AUTH_FILE = "config/authorization_rules.rb"

  class InvalidCurrentUserError < StandardError; end
  class NoAuthorizationContextProvidedError < StandardError; end
  class NoMethodForMatcherError < StandardError; end
  class NotAuthorizedError < StandardError; end
  class MissingMatcherError < StandardError; end
  class MissingPermissionFilterError < StandardError; end

  class << self
    def current_user
      @curent_user ||= nil
      @curent_user || anonymous_user
    end

    def current_user=(user)
      raise InvalidCurrentUserError if !user.nil? && !user.respond_to?(:role_symbols)
      @curent_user = user
    end

    def roles
      @roles ||= build
    end

    def permitted_to?(activity, resource_or_name, options={})
      return true if @without_authorization
      permitted = current_roles.any?{ |r| r.permitted_to?(activity, resource_or_name, options) }
      if options[:bang] && !permitted
        raise NotAuthorizedError.new("#{current_user.to_s} with #{current_user.role_symbols.inspect} is not allowed to #{activity} on #{resource_or_name.inspect}")
      end

      permitted
    end

    def current_roles
      roles.select { |role_symbol, r| current_user.role_symbols.include?(role_symbol) }.values
    end

    def current_permissions_for(activity, resource_or_name)
      current_roles.map do |r|
        r.permissions_for(activity, resource_or_name)
      end.flatten
    end

    def without_authorization(&block)
      @without_authorization = true
      result = block_given? ? yield : nil
      @without_authorization = false
      result
    end

  private

    def build
      Builder.new(authorization_rules).roles
    end

    def authorization_rules
      File.read(AUTH_FILE)
    end

    def anonymous_user
      @anonymous_user ||= AnonymousUser.new
    end
  end
end