module Toni
  module InModel
    def self.included(base)
      base.extend(ClassMethods)

      [:create, :update, :destroy].each do |action|
        base.send(:"before_#{action}") do |object|
          activity = action == :destroy ? :delete : action
          Toni.permitted_to?(activity, object, bang: true)
        end
      end

      base.send(:after_find) do |object|
        Toni.permitted_to?(:read, object, bang: true)
      end

      # TODO: Should be active record only
      base.scope :permitted_to, -> {
        base.send(:filter_by_permissions)
      }
    end

    module ClassMethods

      private

      def filter_by_permissions
        where(*scope_for_filter_permissions)
      end

      def scope_for_filter_permissions
        permissions = Toni.current_permissions_for(:read, authorization_context)
        return ["0 = 1"] if permissions.empty?
        permissions.inject([""]) do |cond, p|
          conditions = p.matchers.map(&:filter_condition).compact.inject([""]) do |cond2, filter_cond|
            concat_filters(cond2, filter_cond)
          end
          concat_filters(cond, conditions, " OR ")
        end
      end

      def concat_filters(cond, conditions, separator=" AND ")
        return cond if conditions[0] == ""
        cond[0] = "#{cond[0]}#{separator unless cond[0].empty?}(#{conditions[0]})"
        cond.push(*conditions[1..-1])
        cond
      end
    end
  end
end