require "toni/permission_matcher"

module Toni
  class Permission

    attr_reader :resource_name, :matchers, :activities

    def initialize(resource_name, activities = [])
      @resource_name  = resource_name
      @matchers       = []
      @activities     = activities
    end

    def add_matcher(method_name, matcher, is_attr = false)
      @matchers << PermissionMatcher::Base.new(method_name, matcher, is_attr)
    end

    def permission_for?(activity, res)
      resource_name == (res.is_a?(Symbol) ? res : res.class.authorization_context) &&
        activities.include?(activity)
    end

    def permitted_to?(activity, res, options={})
      permission_for?(activity, res) && matches?(res)
    end

  private

    def matches?(object)
      return true if object.is_a?(Symbol)
      matchers.all? { |m| m.execute.matches?(object) }
    end

  end
end