module Toni
  class PermissionFilter
    attr_reader :permission_matcher

    def self.permission_filters
      @filterable_matchers ||= constants.select do |c|
        const_get(c).is_a?(Class)
      end
    end

    def initialize(permission_matcher)
      @permission_matcher = permission_matcher
    end

    def filter_condition
      raise "implement me"
    end

    def matcher
      permission_matcher.matcher
    end

    def attr_name
      permission_matcher.method_name
    end

    class Eq < PermissionFilter
      def filter_condition
        ["#{attr_name} = ?", matcher.expected]
      end

      def filter_condition_inverse
        ["#{attr_name} != ?", matcher.expected]
      end
    end

    class BeNil < PermissionFilter
      def filter_condition
        ["#{attr_name} IS NULL"]
      end

      def filter_condition_inverse
        ["#{attr_name} IS NOT NULL"]
      end
    end

    class BeTruthy < PermissionFilter
      def filter_condition
        ["#{attr_name} IS NOT NULL AND #{attr_name} != ''"]
      end

      def filter_condition_inverse
        BeFalsey.new(permission_matcher).filter_condition
      end
    end

    class BeFalsey < PermissionFilter
      def filter_condition
        ["#{attr_name} = '' OR #{attr_name} IS NULL"]
      end

      def filter_condition_inverse
        BeTruthy.new(permission_matcher).filter_condition
      end
    end

    class BeBetween < PermissionFilter
      def filter_condition
        ["#{attr_name} >= ? AND #{attr_name} <= ?", 
          matcher.instance_variable_get(:@min),
          matcher.instance_variable_get(:@max)]
      end

      def filter_condition_inverse
        ["#{attr_name} < ? OR #{attr_name} > ?", 
          matcher.instance_variable_get(:@min),
          matcher.instance_variable_get(:@max)]
      end
    end
  end
end