module Toni
  module PermissionMatcher

    module HasMatcher

      def self.included(base)
        base.send(:attr_reader, :matcher)
      end

      def matches?(object)
        raise "implement me"
      end

      def filter_condition
        return matcher.filter_condition if matcher.respond_to?(:filter_condition)

        filter = permission_filter
        if filter.nil?
          raise Toni::MissingPermissionFilterError.new("No permission matcher for #{matcher.class.inspect}")
        end

        filter.filter_condition
      end

    private

      def permission_filter
        if permission_filter_class
          Toni::PermissionFilter.const_get(permission_filter_class).new(self)
        end
      end

      def permission_filter_class
        @permission_filter ||= Toni::PermissionFilter.permission_filters.detect do |c|
          !matcher.class.name.nil? && c.to_s.split('::').last == matcher.class.name.split('::').last
        end
      end
    end

  end
end