module Toni
  module PermissionMatcher

    class NotMatcher

      include HasMatcher

      def initialize
        @matcher = nil
      end

      def method_missing(method_name, *args)
        builder  = Toni::Builder::ExpectationBuilder.new
        if builder.respond_to?(method_name)
          @matcher = builder.send(method_name, *args)
        else
          super
        end
      end

      def matches?(object)
        raise Toni::MissingMatcherError if @matcher.nil?
        !@matcher.matches?(object)
      end

      def filter_condition
        filter = permission_filter
        unless filter && filter.respond_to?(:filter_condition_inverse)
          raise Toni::MissingPermissionFilterError.new("No permission matcher for #{matcher.class.inspect}")
        end

        filter.filter_condition_inverse
      end
    end

  end
end