module Toni
  class Role

    attr_reader :role_symbol, :ancestors, :permissions

    def initialize(role_symbol)
      @role_symbol  = role_symbol
      @ancestors    = []
      @permissions  = []
    end

    def add_ancestor(role_symbol)
      @ancestors << role_symbol unless @ancestors.include?(role_symbol)
    end

    def add_permission(permission)
      @permissions << permission
    end

    def permitted_to?(activity, res, options={})
      unless res.is_a?(Symbol) || res.class.respond_to?(:authorization_context)
        raise Toni::NoAuthorizationContextProvidedError
      end
      current_permissions.any? do |p|
        p.permitted_to?(activity, res, options)
      end
    end

    def permissions_for(activity, res)
      unless res.is_a?(Symbol) || res.class.respond_to?(:authorization_context)
        raise Toni::NoAuthorizationContextProvidedError
      end
      res = res.class.authorization_context unless res.is_a?(Symbol)
      current_permissions.select { |p| p.permission_for?(activity, res) }
    end

  private

    def current_permissions
      @permissions.concat(ancestor_permissions)
    end

    def ancestor_permissions
      @ancestor_permissions ||= ancestors.map do |ancestor|
        ancestor.permissions
      end.flatten
    end

  end
end